@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -163,6 +166,42 @@ class DatastoreAdminAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return DatastoreAdminClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> DatastoreAdminTransport:
         """Returns the transport used by the client instance.
@@ -228,7 +267,7 @@ class DatastoreAdminAsyncClient:
         request: Union[datastore_admin.ExportEntitiesRequest, dict] = None,
         *,
         project_id: str = None,
-        labels: Sequence[datastore_admin.ExportEntitiesRequest.LabelsEntry] = None,
+        labels: Dict[str, str] = None,
         entity_filter: datastore_admin.EntityFilter = None,
         output_url_prefix: str = None,
         retry: OptionalRetry = gapic_v1.method.DEFAULT,
@@ -246,6 +285,31 @@ class DatastoreAdminAsyncClient:
         before completion it may leave partial data behind in
         Google Cloud Storage.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_export_entities():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.ExportEntitiesRequest(
+                    project_id="project_id_value",
+                    output_url_prefix="output_url_prefix_value",
+                )
+
+                # Make the request
+                operation = client.export_entities(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.ExportEntitiesRequest, dict]):
                 The request object. The request for
@@ -257,7 +321,7 @@ class DatastoreAdminAsyncClient:
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            labels (:class:`Sequence[google.cloud.datastore_admin_v1.types.ExportEntitiesRequest.LabelsEntry]`):
+            labels (:class:`Dict[str, str]`):
                 Client-assigned labels.
                 This corresponds to the ``labels`` field
                 on the ``request`` instance; if ``request`` is provided, this
@@ -313,7 +377,7 @@ class DatastoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any(
             [project_id, labels, entity_filter, output_url_prefix]
@@ -365,7 +429,7 @@ class DatastoreAdminAsyncClient:
         request: Union[datastore_admin.ImportEntitiesRequest, dict] = None,
         *,
         project_id: str = None,
-        labels: Sequence[datastore_admin.ImportEntitiesRequest.LabelsEntry] = None,
+        labels: Dict[str, str] = None,
         input_url: str = None,
         entity_filter: datastore_admin.EntityFilter = None,
         retry: OptionalRetry = gapic_v1.method.DEFAULT,
@@ -380,6 +444,31 @@ class DatastoreAdminAsyncClient:
         is possible that a subset of the data has already been
         imported to Cloud Datastore.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_import_entities():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.ImportEntitiesRequest(
+                    project_id="project_id_value",
+                    input_url="input_url_value",
+                )
+
+                # Make the request
+                operation = client.import_entities(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.ImportEntitiesRequest, dict]):
                 The request object. The request for
@@ -391,7 +480,7 @@ class DatastoreAdminAsyncClient:
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            labels (:class:`Sequence[google.cloud.datastore_admin_v1.types.ImportEntitiesRequest.LabelsEntry]`):
+            labels (:class:`Dict[str, str]`):
                 Client-assigned labels.
                 This corresponds to the ``labels`` field
                 on the ``request`` instance; if ``request`` is provided, this
@@ -453,7 +542,7 @@ class DatastoreAdminAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, labels, input_url, entity_filter])
         if request is not None and has_flattened_params:
@@ -522,6 +611,29 @@ class DatastoreAdminAsyncClient:
 
         Indexes with a single property cannot be created.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_create_index():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.CreateIndexRequest(
+                )
+
+                # Make the request
+                operation = client.create_index(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.CreateIndexRequest, dict]):
                 The request object. The request for
@@ -589,6 +701,29 @@ class DatastoreAdminAsyncClient:
         [delete][google.datastore.admin.v1.DatastoreAdmin.DeleteIndex]
         again.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_delete_index():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.DeleteIndexRequest(
+                )
+
+                # Make the request
+                operation = client.delete_index(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.DeleteIndexRequest, dict]):
                 The request object. The request for
@@ -643,6 +778,24 @@ class DatastoreAdminAsyncClient:
     ) -> index.Index:
         r"""Gets an index.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_get_index():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.GetIndexRequest(
+                )
+
+                # Make the request
+                response = client.get_index(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.GetIndexRequest, dict]):
                 The request object. The request for
@@ -697,6 +850,26 @@ class DatastoreAdminAsyncClient:
         the list of indexes and may occasionally return stale
         results.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_list_indexes():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.ListIndexesRequest(
+                )
+
+                # Make the request
+                page_result = client.list_indexes(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.ListIndexesRequest, dict]):
                 The request object. The request for
