@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,23 +14,25 @@
 # limitations under the License.
 #
 from collections import OrderedDict
-from distutils import util
 import os
 import re
 from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import client_options as client_options_lib
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -275,6 +277,73 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -325,50 +394,22 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        use_client_cert = bool(
-            util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, DatastoreAdminTransport):
             # transport is a DatastoreAdminTransport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -380,6 +421,15 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -397,7 +447,7 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
         request: Union[datastore_admin.ExportEntitiesRequest, dict] = None,
         *,
         project_id: str = None,
-        labels: Sequence[datastore_admin.ExportEntitiesRequest.LabelsEntry] = None,
+        labels: Dict[str, str] = None,
         entity_filter: datastore_admin.EntityFilter = None,
         output_url_prefix: str = None,
         retry: OptionalRetry = gapic_v1.method.DEFAULT,
@@ -415,6 +465,31 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
         before completion it may leave partial data behind in
         Google Cloud Storage.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_export_entities():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.ExportEntitiesRequest(
+                    project_id="project_id_value",
+                    output_url_prefix="output_url_prefix_value",
+                )
+
+                # Make the request
+                operation = client.export_entities(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.ExportEntitiesRequest, dict]):
                 The request object. The request for
@@ -426,7 +501,7 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            labels (Sequence[google.cloud.datastore_admin_v1.types.ExportEntitiesRequest.LabelsEntry]):
+            labels (Dict[str, str]):
                 Client-assigned labels.
                 This corresponds to the ``labels`` field
                 on the ``request`` instance; if ``request`` is provided, this
@@ -482,7 +557,7 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any(
             [project_id, labels, entity_filter, output_url_prefix]
@@ -533,7 +608,7 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
         request: Union[datastore_admin.ImportEntitiesRequest, dict] = None,
         *,
         project_id: str = None,
-        labels: Sequence[datastore_admin.ImportEntitiesRequest.LabelsEntry] = None,
+        labels: Dict[str, str] = None,
         input_url: str = None,
         entity_filter: datastore_admin.EntityFilter = None,
         retry: OptionalRetry = gapic_v1.method.DEFAULT,
@@ -548,6 +623,31 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
         is possible that a subset of the data has already been
         imported to Cloud Datastore.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_import_entities():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.ImportEntitiesRequest(
+                    project_id="project_id_value",
+                    input_url="input_url_value",
+                )
+
+                # Make the request
+                operation = client.import_entities(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.ImportEntitiesRequest, dict]):
                 The request object. The request for
@@ -559,7 +659,7 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
                 This corresponds to the ``project_id`` field
                 on the ``request`` instance; if ``request`` is provided, this
                 should not be set.
-            labels (Sequence[google.cloud.datastore_admin_v1.types.ImportEntitiesRequest.LabelsEntry]):
+            labels (Dict[str, str]):
                 Client-assigned labels.
                 This corresponds to the ``labels`` field
                 on the ``request`` instance; if ``request`` is provided, this
@@ -621,7 +721,7 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, labels, input_url, entity_filter])
         if request is not None and has_flattened_params:
@@ -689,6 +789,29 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
 
         Indexes with a single property cannot be created.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_create_index():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.CreateIndexRequest(
+                )
+
+                # Make the request
+                operation = client.create_index(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.CreateIndexRequest, dict]):
                 The request object. The request for
@@ -757,6 +880,29 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
         [delete][google.datastore.admin.v1.DatastoreAdmin.DeleteIndex]
         again.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_delete_index():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.DeleteIndexRequest(
+                )
+
+                # Make the request
+                operation = client.delete_index(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.DeleteIndexRequest, dict]):
                 The request object. The request for
@@ -812,6 +958,24 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
     ) -> index.Index:
         r"""Gets an index.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_get_index():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.GetIndexRequest(
+                )
+
+                # Make the request
+                response = client.get_index(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.GetIndexRequest, dict]):
                 The request object. The request for
@@ -857,6 +1021,26 @@ class DatastoreAdminClient(metaclass=DatastoreAdminClientMeta):
         the list of indexes and may occasionally return stale
         results.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_admin_v1
+
+            def sample_list_indexes():
+                # Create a client
+                client = datastore_admin_v1.DatastoreAdminClient()
+
+                # Initialize request argument(s)
+                request = datastore_admin_v1.ListIndexesRequest(
+                )
+
+                # Make the request
+                page_result = client.list_indexes(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.datastore_admin_v1.types.ListIndexesRequest, dict]):
                 The request object. The request for
