@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.cloud.datastore_v1.types import datastore
 from google.cloud.datastore_v1.types import entity
@@ -103,6 +106,42 @@ class DatastoreAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return DatastoreClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> DatastoreTransport:
         """Returns the transport used by the client instance.
@@ -176,6 +215,25 @@ class DatastoreAsyncClient:
     ) -> datastore.LookupResponse:
         r"""Looks up entities by key.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_lookup():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.LookupRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.lookup(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.LookupRequest, dict]):
                 The request object. The request for
@@ -212,7 +270,7 @@ class DatastoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, read_options, keys])
         if request is not None and has_flattened_params:
@@ -266,6 +324,25 @@ class DatastoreAsyncClient:
     ) -> datastore.RunQueryResponse:
         r"""Queries for entities.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_run_query():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.RunQueryRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.run_query(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.RunQueryRequest, dict]):
                 The request object. The request for
@@ -320,6 +397,25 @@ class DatastoreAsyncClient:
     ) -> datastore.BeginTransactionResponse:
         r"""Begins a new transaction.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_begin_transaction():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.BeginTransactionRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.begin_transaction(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.BeginTransactionRequest, dict]):
                 The request object. The request for
@@ -344,7 +440,7 @@ class DatastoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id])
         if request is not None and has_flattened_params:
@@ -389,6 +485,27 @@ class DatastoreAsyncClient:
         r"""Commits a transaction, optionally creating, deleting
         or modifying some entities.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_commit():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.CommitRequest(
+                    transaction=b'transaction_blob',
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.commit(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.CommitRequest, dict]):
                 The request object. The request for
@@ -448,7 +565,7 @@ class DatastoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, mode, transaction, mutations])
         if request is not None and has_flattened_params:
@@ -496,6 +613,26 @@ class DatastoreAsyncClient:
     ) -> datastore.RollbackResponse:
         r"""Rolls back a transaction.
 
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_rollback():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.RollbackRequest(
+                    project_id="project_id_value",
+                    transaction=b'transaction_blob',
+                )
+
+                # Make the request
+                response = client.rollback(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.RollbackRequest, dict]):
                 The request object. The request for
@@ -528,7 +665,7 @@ class DatastoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, transaction])
         if request is not None and has_flattened_params:
@@ -573,6 +710,26 @@ class DatastoreAsyncClient:
         r"""Allocates IDs for the given keys, which is useful for
         referencing an entity before it is inserted.
 
+
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_allocate_ids():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.AllocateIdsRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.allocate_ids(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.datastore_v1.types.AllocateIdsRequest, dict]):
                 The request object. The request for
@@ -606,7 +763,7 @@ class DatastoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, keys])
         if request is not None and has_flattened_params:
@@ -648,8 +805,28 @@ class DatastoreAsyncClient:
         timeout: float = None,
         metadata: Sequence[Tuple[str, str]] = (),
     ) -> datastore.ReserveIdsResponse:
-        r"""Prevents the supplied keys' IDs from being auto-
-        llocated by Cloud Datastore.
+        r"""Prevents the supplied keys' IDs from being
+        auto-allocated by Cloud Datastore.
+
+
+        .. code-block:: python
+
+            from google.cloud import datastore_v1
+
+            def sample_reserve_ids():
+                # Create a client
+                client = datastore_v1.DatastoreClient()
+
+                # Initialize request argument(s)
+                request = datastore_v1.ReserveIdsRequest(
+                    project_id="project_id_value",
+                )
+
+                # Make the request
+                response = client.reserve_ids(request=request)
+
+                # Handle the response
+                print(response)
 
         Args:
             request (Union[google.cloud.datastore_v1.types.ReserveIdsRequest, dict]):
@@ -683,7 +860,7 @@ class DatastoreAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([project_id, keys])
         if request is not None and has_flattened_params:
